package com.jjt.hl.service.impl;

import com.jjt.biz.domain.BizModel;
import com.jjt.biz.service.IBizModelDetailService;
import com.jjt.biz.service.IBizModelService;
import com.jjt.common.utils.DateUtils;
import com.jjt.common.utils.bean.BeanUtils;
import com.jjt.hl.domain.*;
import com.jjt.hl.mapper.HlScoreMapper;
import com.jjt.hl.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * 健康度分数记录Service业务层处理
 *
 * @author jjt
 * @date 2024-09-05
 */
@Service
public class HlScoreServiceImpl implements IHlScoreService {
    @Resource
    private HlScoreMapper hlScoreMapper;
    @Resource
    private IBizModelService modelService;
    @Resource
    private IBizModelDetailService modelDetailService;
    @Resource
    private IHlClassService hlClassService;
    @Resource
    private IHlClassScoreService hlClassScoreService;
    @Resource
    private IHlObjService hlObjService;
    @Resource
    private IHlMetricsService hlMetricsService;
    @Resource
    private IHlMetricsScoreService hlMetricsScoreService;
    @Resource
    private IHlMetricsScoreDetailService hlMetricsScoreDetailService;


    /**
     * 查询健康度分数记录
     *
     * @param hlScoreId 健康度分数记录主键
     * @return 健康度分数记录
     */
    @Override

    public HlScore selectHlScoreByHlScoreId(Long hlScoreId) {
        return hlScoreMapper.selectHlScoreByHlScoreId(hlScoreId);
    }

    /**
     * 查询健康度分数记录列表
     *
     * @param hlScore 健康度分数记录
     * @return 健康度分数记录
     */
    @Override
    public List<HlScore> selectHlScoreList(HlScore hlScore) {
        return hlScoreMapper.selectHlScoreList(hlScore);
    }

    /**
     * 新增健康度分数记录
     *
     * @param hlScore 健康度分数记录
     * @return 结果
     */
    @Override
    public int insertHlScore(HlScore hlScore) {
        hlScore.setCreateTime(DateUtils.getNowDate());
        return hlScoreMapper.insertHlScore(hlScore);
    }

    /**
     * 修改健康度分数记录
     *
     * @param hlScore 健康度分数记录
     * @return 结果
     */
    @Override
    public int updateHlScore(HlScore hlScore) {
        hlScore.setUpdateTime(DateUtils.getNowDate());
        return hlScoreMapper.updateHlScore(hlScore);
    }

    /**
     * 批量删除健康度分数记录
     *
     * @param hlScoreIds 需要删除的健康度分数记录主键
     * @return 结果
     */
    @Override
    public int deleteHlScoreByHlScoreIds(Long[] hlScoreIds) {
        return hlScoreMapper.deleteHlScoreByHlScoreIds(hlScoreIds);
    }

    /**
     * 删除健康度分数记录信息
     *
     * @param hlScoreId 健康度分数记录主键
     * @return 结果
     */
    @Override
    public int deleteHlScoreByHlScoreId(Long hlScoreId) {
        return hlScoreMapper.deleteHlScoreByHlScoreId(hlScoreId);
    }

    /**
     * 健康度评分
     */
    @Override
    public void score() {
        //暂时先模拟分数
        //1.获取所有模型
        //2.获取模型健康度指标分类
        //3.1.获取对象
        //3.2.获取指标
        //
        List<BizModel> modelList = modelService.selectBizModelList(new BizModel());
        Date curr = new Date();
        for (BizModel model : modelList) {
            HlScore score = new HlScore();
            score.setModelId(model.getModelId());
            score.setHlDate(curr);
            score.setHlType("1");
            insertHlScore(score);
            List<HlClass> classList = modelService.selectHlClassList4modelId(model.getModelId());
            for (HlClass hlClass : classList) {
                HlClassScore hlClassScore = new HlClassScore();
                BeanUtils.copyBeanProp(hlClassScore, hlClass);
                hlClassScore.setHlScoreId(score.getHlScoreId());
                List<HlObj> hlObjList = hlObjService.selectHlObjListByClassId(hlClass.getHlClassId());
                hlClassScoreService.insertHlClassScore(hlClassScore);
                List<HlMetrics> hlMetricsList = hlMetricsService.selectHlMetricsList4ModelId(hlClassScore.getModelId(), hlClass.getObjType());
                BigDecimal loseScore = BigDecimal.ZERO;
                //1.均分 2.细项分
                hlClass.getScoreType();
                for (HlMetrics hlMetrics : hlMetricsList) {
                    //TODO ..这里需要从对象指标表里面去找。
//                    BizObjMetrics
//                    BizObjMetricsData
                    BigDecimal lose = BigDecimal.ZERO;
                    HlMetricsScore metricsScore = new HlMetricsScore();
                    BeanUtils.copyBeanProp(metricsScore, hlMetrics);
                    if (flag()) {
                        //TODO 模拟扣分判断
                        if ("1".equals(hlClassScore.getScoreType())) {
                            //均分
//                            Random
                        } else {
                            //细项分
                        }
                    }
                }
            }
        }
    }

    /**
     * 日健康度评分
     */
    @Override
    public void yesterday() {

    }

    /**
     * 模拟扣分判断
     *
     * @return
     */
    private boolean flag() {
        Random r = new Random();
        int x = r.nextInt(8889) + 1;
        return x % 2 == 1;
    }
}
